#!/bin/bash
# Copyright (c) Daniel Sheffield 2024 - 2025

set -euo pipefail
if [ "$#" == "0" ]
then
	echo capture
	exit 0
fi

. "${BASH_SOURCE[0]%/*}"/helpers/common.sh

PROG="$1"
SUBPROG="$(get_subprog "$@")"

TIMESTAMP=$(date +%Y%m%d_%H%M%S_%N)
LOG_DIR="$LOG_ROOT"/"$PROG"/"$SUBPROG"/"$TIMESTAMP"

mkdir -p "$LOG_DIR"

OUT="$LOG_DIR"/stdout
ERR="$LOG_DIR"/stderr
DAT="$LOG_DIR"/dat
INF="$LOG_DIR"/info
CMD="$(shell_escape "$@")"

info(){
	cat <<EOF > "$INF"
$CMD
status=$?
EOF
	exec 1>&- 2>&-
	[ "${socat:-}" ] && kill "$socat"
	wait
}

trap 'info' EXIT

exec 2> >(tee -a "$DAT" "$ERR")
if [ -t 1 ] && ! [ "$PROG" = "sudo" ] && which socat 1>/dev/null 2>&1
then
	PTY="$LOG_DIR"/pty
	socat PTY,rawer,link="$PTY"A PTY,rawer,link="$PTY"B &
	socat="$!"
	for pty in "$PTY"{A,B}
	do
		while ! [ -e "$pty" ]; do sleep 0.01; done
		stty -F "$pty" rows "$LINES" cols "$COLUMNS"
	done
	tee -a "$DAT" "$OUT" 2>/dev/null < "$PTY"B &
	exec 1> "$PTY"A
else
	exec 1> >(tee -a "$DAT" "$OUT")
fi
"$@"
